function [x,w,we] =glaxw_spec(n)
% glaxw  Nodes and weights for Gauss-Laguerre quadrature of arbitrary order
%             by solving an eigenvalue problem
%
% Synopsis:  [x,w,we] = glwax_spec(n)
%
% Input:     n = order of quadrature rule
%
% Output:    x = vector of nodes
%            w = vector of weights
%           we = vector of weight mutliplied by exp(x)

%  Algorithm based on ideas from Golub and Welsch, and Gautschi.  For a
%  condensed presentation see H.R. Schwarz, "Numerical Analysis: A
%  Comprehensive Introduction", 1989, Wiley.  Original MATLAB
%  implementation by H.W. Wilson and L.H. Turcotte, "Advanced Mathematics
%  and Mechanics Applications Using MATLAB", 2nd ed., 1998, CRC Press

%[D,V]  = tde(1:2:2*n-1,-(1:n-1));
[D,V]  = trideigs(1:2:2*n-1,-(1:n-1));

x = D(:);  %  nodes are eigenvalues, which are on diagonal of D
w      =   V(1,:)'.^2;  %  V(1,:)' is column vector of first row of sorted V
we     = exp(x+2*log(abs(V(1,:)')));
