#include "mex.h"
#include <string.h>
#include <stdio.h>
#include "fort.h"
#include "stddef.h"
/*   TRIDEIGs Eigenvalues an egien vectors of symmetric tridiagonal matrix.   */

/*   V. Yordanov <v.yordanov@phys.uni-sofia.bg>   */
/*   Department of Optics and Spectroscopy, Sofia University    */
/*   May, 2009                  */
/*  [W] = trideigs(D,E) */
/*  [W,Z] = trideigs(D,E) */
/*  [W,Z] = trideigs(D,E,RANGE,VL,VU) */
/*  [W,Z] = trideigs(D,E,RANGE,IL,IU) */

/* Quick install */
/* For windows: */
/* %MATLAB\bin\mex trideigs.c %MATLAB\extern\lib\win32\lcc\libmwlapack.lib */
/* For linux: */
/* $MATLAB/bin/mex -llapack trideigs.c */
/* Add the directory that contains compiled mex file to your path. */
/* Add tridiegs.m to the path also. */


void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
{
  double *D,*E,VL,VU,*WORK,ABSTOL,*W,*Z;
  ptrdiff_t  N,IL,IU,M,*IWORK,LDZ,*IFAIL,INFO;
  mxArray *W_arr,*Z_arr;
  char JOBZ[2]={'V','\0'},RANGE[2]={'A','\0'};

  if (nrhs>5)
    mexErrMsgTxt("Must have max 5 input arguments.");
  if (nrhs<2)
    mexErrMsgTxt("Must have min 2 input arguments.");
  if (nlhs>2)
    mexErrMsgTxt("Too many output arguments.");
  if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]))
    mexErrMsgTxt("Both inputs must be double arrays.");
  if (mxIsComplex(prhs[0]) || mxIsComplex(prhs[1]))
    mexErrMsgTxt("Both inputs must be real.");
  if (mxGetM(prhs[0])!=1 && mxGetN(prhs[0])!=1)
    mexErrMsgTxt("First input must be a vector.");
  if (mxGetM(prhs[1])!=1 && mxGetN(prhs[1])!=1)
    mexErrMsgTxt("Second input must be a vector.");
  if (mxGetM(prhs[1])!=1 && mxGetN(prhs[1])!=1)
    mexErrMsgTxt("Second input must be a vector.");

  N=(ptrdiff_t) mxGetNumberOfElements(prhs[0]);

  if (mxGetNumberOfElements(prhs[1])!=N-1)
    mexErrMsgTxt("The input vectors must have length N and N-1.");

  
  D=mxGetPr(prhs[0]);
  E=mxGetPr(prhs[1]);
  M=N; /* default value */
  VL=-1.0e100; VU=1.0e100; /* default values */
  IL=1; IU=N; /* default values */

  
 
  LDZ=N;
  W_arr = mxCreateDoubleMatrix(N,1, mxREAL);
  W=(double*) mxGetPr(W_arr);
  Z_arr = mxCreateDoubleMatrix(N,N,mxREAL);
  Z=(double*) mxGetPr(Z_arr);
  WORK=(double*) mxCalloc(10*N,sizeof(double));
  /*  IWORK=(ptrdiff_t*) mxCalloc(5*N,sizeof(int));
      IFAIL=(ptrdiff_t*) mxCalloc(N,sizeof(int));*/
  IWORK=mxCalloc(10*N,sizeof(int));
  IFAIL=mxCalloc(2*N,sizeof(int));
  ABSTOL=-0.1; (double) 2.0*dlamch_("S");
  dstevx_("V","A",&N,D,E,&VL,&VU,&IL,&IU,&ABSTOL,&M,W,Z,&LDZ,WORK,IWORK,IFAIL,&INFO);
  mxFree(IFAIL);
  mxFree(IWORK);
  mxFree(WORK); 
  plhs[0]=W_arr;
  plhs[1]=Z_arr;
 
  if (INFO>0)
    mexErrMsgTxt("No convergence in DSTEVX.");
}
